.comment CS Library Manual
.comment Version V02.00
.comment 20-Oct-82
.comment
.ps 57,80
.lm 8
.rm 72
.p 3,1,3
.ap
.hy
.hd mixed
.s 6
.c ;CS Library User's Manual
.s 2
.c ;Written by:
.c ;Timothy T. Coad
.s 2
.c ;Release V02.00
.c ;October 1982
.s 3
.ch Introduction
.title #######CS Library
.ps 57,80
.lm 8
.rm 72
.p 3,1,3
.ap
.hy
.hd mixed

    This document describes the CS library routines for the
DECUS C programming system. It assumes that the user is familiar
with the C language.

    The CS library provides a stand alone subset of the complete
C language system run-time support library along with a
convenient run-time interface to EXPRESS, the RSX-11M/M+ File
Server ACP, from C. Aside from the EXPRESS file I/O support,
none of the standard C I/O services are supported with CS. This
was done so that the size of a C program which did not need this
support would be considerably reduced.  The CS library does
contain its own terminal I/O routines.

    The SMAIN object module is also available for the user to
even further reduce the size of a C program. The SMAIN object
module provides for a separate start up entry point which does
not support dynamic memory allocation, command line processing,
the fancy CS trap catching or the ability to dynamically set the
internal output buffer.  If an application program does not
require any of the above features then the use of SMAIN is
recommended.

    There is one header file, CS.H, which defines the FSB for an
EXPRESS file and contains #define's for recording mode values,
file format values, and key flags. CS.H also defines as external
the directive status word and EXPRESS I/O status block, $DSW and
f_iosb, in order to allow the user to examine them upon completion
of an EXPRESS function call.

.ch CS Library Routines
.title ########CS Library
.ps 57,80
.lm 8
.rm 72
.p 3,1,3
.ap
.hy
.hd mixed
.lm 8.rm 72.nhy

.st ########$$c5ta##Convert Radix-50 to Ascii
.pg
.hl 1 ^&Convert Radix-50 to Ascii\&
.s 2
.c ;**********
.c ;* $$c5ta *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
mov	rad50__word, r1
mov	out__ptr, r0
call	$$c5ta
.s
.i -8;Description
.s
.fill
Convert one radix-50 word to three Ascii bytes. All letters
will be in upper-case.  The output buffer will not be null
trailed, nor will blank fields be supressed.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$gcmd##Parse command line
.pg
.hl 1 ^&Parse command line\&
.s 2
.c ;**********
.c ;* $$gcmd *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
argc = $$gcmd(text, avp, task)
char *text;     /* Command line */
char ***avp;    /* Gets argv[] location */
char *task;     /* Task name if nonnull */
.s
.i -8;Description
.s
.fill
Parse the command line and build the argv[] table.
.s
NOTE: $$gcmd will modify text.  argv[] entries will point
to bytes in text.  If not NULL, the task name will become
argv[0].
.s
$$gcmd() returns the number of arguments encountered.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$init##One-time initialization code
.pg
.hl 1 ^&One-time initialization code\&
.s 2
.c ;**********
.c ;* $$init *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
$$init()
.s
.i -8;Description
.s
.fill
This is a one-time initialization routine for C programs. It
sets up the trap vectors, initializes the free memory pointers,
determines the task name, assigns the terminal lun to 1, prompts
for a command line if one was not given and proceeds to parse
the command line if one was given and form the argv[] array.
argv[0] is set to the task name.
.s
.i -8;Diagnostics
.s
No memory
.s
.nf
   This message suggests a severe case of program
   immensity.
.s
.fill
Cannot parse command line
.s
.nf
   The user supplied an illegal command line.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$ltoa##Convert long to Ascii
.pg
.hl 1 ^&Convert long to Ascii\&
.s 2
.c ;**********
.c ;* $$ltoa *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
$$ltoa(buffer, radix, value);
char *buffer;   /* Where it goes */
int  radix;     /* Conversion radix */
long value;     /* What to convert */
.s
.i -8;Description
.s
.fill
Called by format to convert a long integer from binary to
ascii, storing the result in buffer.  The radix argument
determines the conversion:
.s
.nf
	d	Signed decimal
	o	Octal
	u	Unsigned decimal
	X	Hex (10-15 = A-F)
	x	Hex (10-15 = a-f)
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$main##Start of C programs
.pg
.hl 1 ^&Start of C programs\&
.s 2
.c ;**********
.c ;* $$main *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
$$main::		/* Start of C programs	*/
.s
extern int   $$uic;	/* RSX: Default uic	*/
extern int   $$stnd;	/* Stack end point	*/
extern char  $$task[];	/* RSX: task name	*/
.s
.i -8;Description
.s
.fill
This module contains the run-time startoff for C programs.
It initializes the C environment and calls the main() subroutine.
On return from the main program, exit() is called to flush the
output buffer and return to the operating system. The following
is a list of internal symbols used by the C run-time.
.s
.nf
	$$stnd	The (top) end of the stack on entrance.
	$$task	The task name in ascii.
	$$uic	The default UIC word from GTSK$.
	$$mend	End of untouched memory
	$$argc	Number of command args
	$$argv	Pointer to argv array
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$narg##Define $$narg default
.pg
.hl 1 ^&Define $$narg default\&
.s 2
.c ;**********
.c ;* $$narg *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
int	$$narg	= <value>;
_...
main()
   {
	...
   }
.s
.i -8;Description
.s
.fill
If no command line is provided when the program starts,
the program will print a command line prompt on the terminal,
read one line and build the argv[] array. If this is
undesirable, include a global definition of $$narg in your C
program as follows:
.s
.nf
	int $$narg = 1;
.s
.fill
This will supress the terminal read operation and the argv[]
array will be set to a dummy value.
.s
If the user program does not define $$narg, it defaults to 0
which enables the command line prompt.
.s
Note that $$narg must be initilized at compilation time.
It is tested by the initialization code before the main()
routine is called.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########$$prmt##Command line prompt string pointer
.pg
.hl 1 ^&Command line prompt string pointer\&
.s 2
.c ;**********
.c ;* $$prmt *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *$$prmt = "Prompt string";
.s
.i -8;Description
.s
.fill
This string pointer may be used to define ones own command line
prompt. If this is not defined by the user then the default
prompt (task name) will appear.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########.xqiow##Execute a QIO
.pg
.hl 1 ^&Execute a QIO\&
.s 2
.c ;**********
.c ;* .xqiow *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
mov	_#qiodpb,r0
call	.xqiow
.s
.i -8;Description
.s
.i -8;##	Executes FILES-11 QIOs with wait and auto retry on pool
.fill
exhaustion. Assumes waiting on event flag.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########acsv$###C register save and restore
.pg
.hl 1 ^&C register save and restore\&
.s 2
.c ;*********
.c ;* acsv$ *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
jsr	r5,csv$
_...
jmp	cret$
.s
.i -8;Description
.s
.fill
Each C subroutine starts with a call to CSV$ and exits by
jumping to CRET$.  Upon exit, the stack need not be equal
to its value on entrance.  This version preserves R0, so
that AST's will work.
.s
During the execution of all C subroutines,  register R5
points to the current "environment."  Within a subroutine,
it appears as follows:
.s
.nf
	 ______________________________
	|		|
  SP ->	| 1st loc. var. | -10(R5)	C$AUTO-2(R5)
	|______________________________|
	|		|
	| Saved R2	|  -6(R5)
	|______________________________|
	|		|
	| Saved R3	|  -4(R5)
	|______________________________|
	|		|
	| Saved R4	|  -2(R5)
	|______________________________|
	|		|
  R5 ->	| Saved R5	|
	|______________________________|
	|		|
	| Return add.	|  +2(R5)
	|______________________________|
	|		|
	| First arg.	|  +4(R5)	C$PMTR+0(R5)
	|______________________________|
	|		|
	| Second arg.	|  +6(R5)	C$PMTR+2(R5)
	|______________________________|
.s
.fill
Within a subroutine, Registers R0-R4 and the top of the
stack, (sp), are available for use.  Registers R0 and R1
are not preserved by subroutines and may be used to pass
a return value.
.s
R5 must not be modified by a subroutine. All variable
addressing must be done by offsets to R5.  Subroutine
arguments must be accessed by reference to C$PMTR.
Subroutine local variables must be accessed by reference to
C$AUTO.  This permits modification of calling sequences
without rewriting all subroutines.
.s
CSV$ refers to global symbol $$main to call the run-time
startup program from the (RSX) library.
.s
An RSX AST routine must call astset() immediately upon
entry to save R0 and R1.  These registers are restored by
astx(), which must be used to exit from an AST service
routine.
.s
See the documentation of the CX library, CX.DOC.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########error###Error exit from C programs
.pg
.hl 1 ^&Error exit from C programs\&
.s 2
.c ;*********
.c ;* error *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
error(format, arglist);
.s
.i -8;Description
.s
.fill
An error message is written to the terminal and the program
exits.  The exit status ($$exst) will be set to "error".
.lm -8.fill
.lm 8.rm 72.nhy

.st ########exit####Exit from C programs
.pg
.hl 1 ^&Exit from C programs\&
.s 2
.c ;********
.c ;* exit *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
exit();			/* Exit, no return */
exits(status);		/* Exit with status */
$$fail();		/* Immediate exit */
.s
extern int $$exst;	/* Exit status value */
.s
.i -8;####Internal
.s
jmp	$$exit		;Call exit routine
.s
.i -8;Description
.s
.fill
On exit, the following sequence occurs:
.nf
  1. The internal output buffer is flushed.
  2. The user-supplied wrapup() function is called.
  3. The program exits to the operating system.
.s
.fill
By calling exits() with an appropriate value, the program can
pass an exit status code to the operating system monitor.
.s
.nf
	RSX	Meaning
	 1.	Success -- no message is printed
	 0.	Warning
	 2.	Error
	 4.	Severe error
.s
.fill
Calling exit() is equivalent to calling exits(1). Note that
the program may also set a value into $$exst and exit via
exit():
.s
.nf
	extern int $$exst;
.s
	...
.s
	$$exst = 4;
	exit();
.s
.fill
If the program calls, or jumps to $$fail, it will immediately
exit to the operating system without calling wrapup().
.lm -8.fill
.lm 8.rm 72.nhy

.st ########flush###Flush the output buffer
.pg
.hl 1 ^&Flush the output buffer\&
.s 2
.c ;*********
.c ;* flush *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
flush();
.s
.i -8;Description
.s
.fill
flush() will force the internal output buffer to be flushed.
This function actually issues the QIO for all of the terminal
I/O functions in the CS library. It returns 0 if an error occurs
and 1 if successful.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########format##- Format a string UNIX style
.pg
.hl 1 ^&- Format a string UNIX style\&
.s 2
.c ;**********
.c ;* format *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
length = format(buffer, format, arg1, ...);
char *buffer;
char *format;
int length;
.s
.i -8;Description
.s
.fill
format() converts and formats its arguments, under the control
of the format string, into the buffer indicated in the first
argument.
.s
The format string is a character string which contains two
types of objects: plain characters, which are simply copied
to the buffer, and conversion specifications, each of which
cause conversion of the next successive argument.
.s
Each conversion specification is introduced by the _% character.
Following the _%, there may be
.s
.nf
 - an  optional  minus  sign  "-"  which  specifies left
   adjustment of the converted argument in the indicated
   field.
.s
 - an optional digit string specifying field  width;  if
   the  converted argument has fewer characters than the
   field width, it will be blank-padded on the left  (or
   right, if  the  left-adjustment  indicator  has  been
   given) to make up the  field  width.   If  the  field
   width  is  specified as '*' or '?', the next argument
   is used.  (Note: '?' is obsolete.)
.s
 - an  optional  period "." which serves to separate the
   field width from the next digit string;
.s
 - an optional digit string (precision) which  specifies
   the number of digits  to  appear  after  the  decimal
   point for e- and f-conversion, or the maximum  number
   of  characters  to be printed from a string.   If the
   precision is  specified  as  '*'  or  '?',  the  next
   argument is used.
.s
 - a character which indicates the  type  of  conversion
   to be applied.
.s
.fill
The conversion characters and their meanings are
.s
.nf
 d	Signed-decimal
 u	Unsigned-decimal
 o	Octal
 X	Hexadecimal, 10-15 are represented by A-F
 x	Hexadecimal, 10-15 are represented by a-f
.s
	The  integer  argument  is converted to decimal,
	octal, or hexadecimal   notation   respectively.
	Any   of   the   conversion  characters  may  be
	preceeded  by  'l'  to  signal  "long"   integer
	argument.   Note  that the Unix usage of capital
	letters  to  represent  long  arguments  is  not
	supported.
.s
 c	The argument character is printed.   (Note  that
	'lc' takes a long integer argument.)
.s
 r	Remote format.  The next  printf()  argument  is
	the format.  Note that this is not a subroutine.
	The current format is not processed further. For
	example:
.s
		bug(args)
		{
			error("Error at _%r", _&args);
		}
.s
	This routine might be called as follows:
.s
		bug("Error _%d at _%s_\n", val, name);
.s
 s	The  argument is taken to be a string (character
	pointer) and  characters  from  the  string  are
	printed  until  a  null  character  or until the
	number of characters indicated by the  precision
	specification  is   reached;   however   if  the
	precision   specification  is  0  or missing all
	characters up to null are printed.
.s
.fill
If no recognizable character appears after the _%, that character
is printed; thus _% may be printed by the use of the string _%_%.
In no case does a non-existant or small field width cause truncation
of a field; padding takes place only if the specified field
width exceeds the actual width.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########malloc##Allocate and free memory
.pg
.hl 1 ^&Allocate and free memory\&
.s 2
.c ;**********
.c ;* malloc *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
char *
malloc(size);     /* NULL if no space */
unsigned size;    /* Number of bytes  */
.s
mfree(p);
free(p);
char *p;          /* Was allocated */
.s
.i -8;Description
.s
.fill
malloc() allocates the indicated number of bytes, returning
a pointer to the first.  If the allocation request cannot
be satisfied, malloc returns NULL. The program image is
expanded as needed.
.s
free() and mfree() return a buffer to free space. Nothing
is returned from the routines.
.s
See also the description of sbreak().
.s
.i -8;Diagnostics
.s
The program may abort if free() is passed a random pointer of
if the program writes outside of an area reserved by malloc().
Note that only certain cases are trapped by this test.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########memdmp##Dump memory or registers
.pg
.hl 1 ^&Dump memory or registers\&
.s 2
.c ;**********
.c ;* memdmp *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
memdmp(start, end);	/* Memory dump routine	*/
char *start;            /* First to dump */
char *end;              /* Last to dump */
.s
regdmp();		/* Register dump */
.s
.i -8;Description
.s
.fill
Dump registers and/or memory.  All registers are preserved.
.s
If memdmp is called with a zero argument, the stack will
be dumped.
.s
.i -8;Bugs
.s
Condition codes are not preserved.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########opncre##Open and create files
.pg
.hl 1 ^&Open and create files\&
.s 2
.c ;**********
.c ;* opncre *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
stream = ropen(fsb, fsb__size, cmode, dprot, options)
int stream;      /* stream number */
char *fsb;	 /* pointer to fsb */
int  fsb__size;   /* size of fsb */
char *cmode;	 /* connect mode */
char *dprot;	 /* dynamic protection */
char *options;	 /* string of options */
.s
.s
stream = rcreat(fsb, fsb__size, cmode, dprot, options)
int stream;      /* stream number */
char *fsb;	 /* pointer to fsb */
int  fsb__size;   /* size of fsb */
char *cmode;	 /* connect mode */
char *dprot;	 /* dynamic protection */
char *options;	 /* string of options */
.s
.s
.i -8;Description
.s
.fill
ropen() connects a file with a stream number and opens it
according to the fsb. It returns the stream number on success
and 0 on error. The stream number may then be used on subsequent
file server function calls as it identifies the connection.
.s
A file may be opened with different connect and dynamic protection
modes.  The following is a list of the possible connect and dynamic
protection modes:
.s
.nf
	Connect:
     	   i - open for input only
           o - open for output only
    	  io - open for input and output
     	   a - open for append
.s
	Dynamic protection:
     	   s - The file may be connected to by any
	       other user (with proper static privilege)
	       for either INPUT, I/O or APPEND. This is
	       the "multiple writers" mode.
     	   p - The file may be connected to by any other
	       user (with proper static privilege) for
	       INPUT only. This is the "single writer
	       multiple readers" mode.
           e - The file may not be connected to by any
	       other user.
.s
.fill
The options available with the ropen() function are as follows:
.s
.i -8;##		Options:
.nf
     	   d - auto-delete on disconnect if dprot
	       EXCLUSIVE
     	   n - do not auto-extend
     	   w - disable write thru caching
	       (for writes)
.s
.s
.fill
rcreat() connects a file with a stream number and creates a new file
according to the fsb. It returns the stream number on success and
and 0 on error. The stream number may then be used on subsequent
file server function calls as it identifies the connection.
.s
A file may be created with different connect and dynamic protection
modes.  The following is a list of the possible connect and dynamic
protection modes:
.s
.nf
	Connect:
     	   i - open for input only
     	   o - open for output only
    	  io - open for input and output
     	   a - open for append
.s
	Dynamic protection:
     	   s - The file may be connected to by any
	       other user (with proper static privilege)
	       for either INPUT, I/O or APPEND. This is
	       the "multiple writers" mode.
     	   p - The file may be connected to by any other
	       user (with proper static privilege) for
	       INPUT only. This is the "single writer
	       multiple readers" mode.
           e - The file may not be connected to by any
	       other user.
.s
.fill
The options available with the rcreat() function are as follows:
.s
.i -8;##		Options:
.nf
     	   d - auto-delete on disconnect if dprot
	       EXCLUSIVE
     	   n - do not auto-extend
     	   s - supersede existing file if same version
     	   w - disable write thru caching
	       (for writes)
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########outbuf##Change output buffer size
.pg
.hl 1 ^&Change output buffer size\&
.s 2
.c ;**********
.c ;* outbuf *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
result = outbuf(size);
int result;
unsigned size;
.s
.i -8;Description
.s
.fill
When a C program is started, the output buffer is allocated
to a default size of 132 decimal characters. A program may
call this routine to reset the output buffer size. This routine
sets up the output buffer pointers and returns a 1 if all is
successful and a 0 on error.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########printf##Formatted print routine
.pg
.hl 1 ^&Formatted print routine\&
.s 2
.c ;**********
.c ;* printf *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
printf(format, arg1, ...);
char *format;
.s
printnf(format, arg1, ...);
char *format;
.s
.i -8;Description
.s
.fill
printf() and printnf convert, format, and print their
arguments, under control of the first argument. printf()
causes the users internal output buffer to be flushed,
whereas printnf() does not flush it.
.s
See description in format.mac for further details on the
format argument.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########prmclr##Clear parameter list
.pg
.hl 1 ^&Clear parameter list\&
.s 2
.c ;**********
.c ;* prmclr *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
mov	_#parmlist,r0
call	prmclr
.s
.i -8;Descriptions
.s
.fill
Clears all six paramters in a qio parameter list.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########qiodef##Qio definitions
.pg
.hl 1 ^&Qio definitions\&
.s 2
.c ;**********
.c ;* qiodef *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;Description
.s
.fill
Header file for CS library routines. Defines the terminal
and file server QIO directive parameter blocks, LUNs, IOSBs
and event flags.
.s
.nf
	Terminal lun            tty.lun == 1
	File server LUN         fs.lun  == 5
	I/O event flag          eflag   == 2
.s
	Terminal QIO IOSB       t.iosb
	File Server IOSB        f.iosb
.s
	Terminal QIO DPB        ttqio
	File Server QIO DPB     fsqio
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########r50toa##Convert Radix-50 to Ascii
.pg
.hl 1 ^&Convert Radix-50 to Ascii\&
.s 2
.c ;**********
.c ;* r50toa *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
r50toa(buff, r5vec, r5cnt);
char *buff;   /* Output text buffer */
int  *r5vec;  /* Input rad50 buffer */
int  r5cnt;   /* How many rad50 words */
.s
.i -8;Description
.s
.fill
Convert r5cnt words of radix 50 data to Ascii.  All letters
will be in upper-case.  The output buffer will not be
null-trailed, nor will blank fields be supressed.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rclose##Disconnect a stream number
.pg
.hl 1 ^&Disconnect a stream number\&
.s 2
.c ;**********
.c ;* rclose *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
rclose(stream, options)
int stream;          /* Stream number to disconnect */
char *options;       /* String of options */
.s
.i -8;Description
.s
.fill
rclose() disconnects a stream number from a file. It returns 1
if successful and 0 if an error occurred.
.s
.i -8;##	Options:
.nf
     m - mark file for delete on final
	 stream disconnect.
     s - send file to spooler if dprot
	 is PROTECTED or EXCLUSIVE and file
	 is ASCII sequential.
     t - truncate file to logical end-of-file
	 if dprot is EXCLUSIVE.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rdelet##Delete a file
.pg
.hl 1 ^&Delete a file\&
.s 2
.c ;**********
.c ;* rdelet *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
rdelet(fsb, fsb__size, options)
char *fsb;	/* Address of fsb */
int  fsb__size;	/* size of fsb (bytes) */
char *options;  /* String of options */
.s
.i -8;Description
.s
.fill
rdelet() attempts to delete the file specified in the fsb.
rdelet() returns 1 if the delete was successful and 0 if an
error occurred. An error may occur if the file is locked or
is currently connected to another record stream.
.s
.i -8;##	Options:
.nf
     w - wait until all streams are disconnected
	 then delete
     m - mark file for delete on final stream
	 disconnect
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rfs#####Return file information
.pg
.hl 1 ^&Return file information\&
.s 2
.c ;*******
.c ;* rfs *
.c ;*******
.s 2
.lm +8.nf
.i -8;Usage
.s
rfs(fsb, fsb__size)
char *fsb;		/* Address of fsb */
int fsb__size;		/* Size of fsb */
.s
.i -8;Description
.s
.fill
rfs() allows the user to obtain information about a file. The user
must pass the address of a fsb and fill in the file name information
then issue the rfs() call. The fsb will be filled in with the full
file specification information. rfs() returns a 0 on an error and
and a 1 on success.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rmark###Mark the file position
.pg
.hl 1 ^&Mark the file position\&
.s 2
.c ;*********
.c ;* rmark *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
extern long rmark();
.s
fpb = rmark(stream)
long fpb;       /* File position */
int stream;     /* Stream number returned by open */
.s
.i -8;Description
.s
.fill
rmark() is used in conjunction with seek() to achieve a
degree of random access capability in sequential files.
rmark() returns -1 on error and the file position on success.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rpos####Position file on key
.pg
.hl 1 ^&Position file on key\&
.s 2
.c ;********
.c ;* rpos *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
rpos(stream, keybuf, keysize, options [, kor])
int stream;	/* Stream number returned by open */
char *keybuf;	/* Key to search on */
int keysize;	/* Size of key */
char *options;	/* String of options */
int kor;	/* New key-of-reference number */
		/* (if "s" option was given) */
.s
.i -8;Description
.s
.fill
rpos() sets the stream position in a file, given a key value.
It may be issued only for relative and indexed files. It returns
0 on error and 1 on success. The options available with the
rpos() function are as follows:
.s
.i -8;##		Options:
.nf
     	   g  - position greater than key
     	   n  - position greater than or equal to key
     	   s  - set key of reference
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rput####Put a record
.pg
.hl 1 ^&Put a record\&
.s 2
.c ;********
.c ;* rput *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
rput(stream, buffer, bufsize, keybuf, keybufsz)
int stream;      /* Stream number from open/create */
char *buffer;    /* Record to transfer */
int bufsize;     /* Size of the buffer */
char *keybuf;    /* Key buffer */
int keybufsz;    /* Size of key buffer */
.s
.i -8;Description
.s
.fill
rput() transfers the contents of the user record buffer into
a new record in the file. It returns 0 on error and 1 on
success.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rrwd####Rewind a stream pointer
.pg
.hl 1 ^&Rewind a stream pointer\&
.s 2
.c ;********
.c ;* rrwd *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
rrwd(stream, options [, kor])
int stream;    /* Stream number from open/create */
char *options; /* Option string */
int kor;       /* New key-of-reference number */
	       /* (if "s" option was given) */
.s
.i -8;Description
.s
.fill
rwd() resets the stream position to the beginning of the file.
It returns 0 on error and 1 on success. The following option is
available for ISAM files only.
.s
.nf
	Option:
	   s - set key of reference
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rseek###Seek to a file position
.pg
.hl 1 ^&Seek to a file position\&
.s 2
.c ;*********
.c ;* rseek *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
rseek(stream, _&fpb)
int stream;	/* Stream number from open/create */
long fpb;	/* Seek position */
.s
.i -8;Description
.s
.fill
rseek() sets the stream position of a sequential file to the
supplied file position. It is used in conjunction with mark() to
achieve a degree of random access capability in sequential files.
It returns a 0 on error and 1 on success.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########runlck##Unlock all records on a stream
.pg
.hl 1 ^&Unlock all records on a stream\&
.s 2
.c ;**********
.c ;* runlck *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
runlck(stream)
int stream;   /* Stream number from open/create */
.s
.i -8;Description
.s
.fill
ulk() unlocks all records which may have been lock on the
given stream number. It returns a 0 on error and 1 on success.
.lm -8.fill
.lm 8.rm 72.nhy

.st ########rwgter##Rewrite, get and erase records
.pg
.hl 1 ^&Rewrite, get and erase records\&
.s 2
.c ;**********
.c ;* rwgter *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
rewrit(stream, buffer, size, key, keysize, options)
int stream;      /* Stream number from open/create */
char *buffer;    /* Buffer containing updated info */
int size;        /* Size of the updated buffer */
char *key;       /* Key to search on */
int keysize;     /* Size of key buffer */
char *options;   /* String of options */
.s
.s
rget(stream, buffer, size, key, keysize, options [, kor)
int stream;      /* Stream number from open/create */
char *buffer;    /* Buffer to copy record into */
int size;        /* Maximum number of bytes to read */
char *key;       /* Key to search for */
int keysize;     /* Size of key buffer */
char *options;   /* String of options */
int kor;	 /* New key-of-reference number */
		 /* (if "s" option was given) */
.s
.s
rerase(stream, buffer, size, key, keysize, options)
int stream;      /* Stream number from open/create */
char *buffer;    /* Record to delete */
int size;        /* Size of the buffer */
char *key;       /* Key to search on */
int keysize;     /* Size of key buffer */
char *options;   /* String of options */
.s
.s
.i -8;Description
.s
.fill
rewrit() updates an existing record. It is valid only for
relative and indexed files. It returns a -1 on error and a 1
on success. The following option is available:
.s
.i -8;##		Options:
.nf
     	   w - wait on locked record
.s
.fill
rget() reads the contents of an existing record into the user
buffer. It returns -1 on error or EOF and the actual size of
the record on success. The following options are available:
.s
.i -8;##		Options:
.nf
     	   l - lock record on read
	   w - wait on locked record
	   s - set new key of reference (ISAM only)
.s
.fill
rerase() erases an existing record. It is valid only for binary
sequential, relative and indexed files. It returns -1 on error
and 1 on success. The following option is available:
.s
.i -8;##		Options:
.nf
           w - wait on locked record
.lm -8.fill
.lm 8.rm 72.nhy

.st ########sbreak##Allocate memory from operating system
.pg
.hl 1 ^&Allocate memory from operating system\&
.s 2
.c ;**********
.c ;* sbreak *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
.i -8;#####Internal
.s
char *
sbreak(amount);
int   amount;
.s
.i -8;Description
.s
.fill
sbreak() allocates the indicated amount of memory from
the operating system.  The allocation is permanent.
.s
.i -8;Diagnostics
.s
NULL is returned if the memory requested is not available.
There is no "shortened" return.
.s
sbreak() does not modify its argument.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########smain###Small main
.pg
.hl 1 ^&Small main\&
.s 2
.c ;*********
.c ;* smain *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
When task build, smain must be explicitly specified so
that it is used instead of the main in the library.
.s
.i -8;Description
.s
.fill
Small main may used to generate smaller task images for
tasks that do not use or do not wish to use dynamic
memory allocation, the command line, the command line prompt
string and the fancy trap catching. This version of main
also maintains a static 132. byte output buffer.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ttin####Terminal input
.pg
.hl 1 ^&Terminal input\&
.s 2
.c ;********
.c ;* ttin *
.c ;********
.s 2
.lm +8.nf
.i -8;Usage
.s
length = ttise(buffer, size);
int length;		/* Number of bytes read */
char *buffer;   	/* Address of input buffer */
int size;		/* size of input buffer */
.s
.s
length = ttisne(buffer, size);
int length;		/* Number of bytes read */
char *buffer;   	/* Address of input buffer */
int size;		/* size of input buffer */
.s
.s
extern char ttice();
ch = ttice();
char ch;		/* returned character */
.s
.s
extern char tticne();
ch = tticne();
char ch;		/* returned character */
.s
.s
.i -8;Description
.s
.fill
ttise() reads a maximum of "size" characters into "buffer"
echoing them on the terminal, and appends a null  (see the
NOTE below!) to the end of the just read string.
.s
ttisne() reads a maximum of "size" characters into "buffer"
without echoing them on the terminal, and appends a null
(see the NOTE below!) to the end of the just read string.
.s
Both ttise() and ttisne() return the number of characters
read (excluding the terminator such as <CR>) or 0 if
there was an error.
.s
NOTE: The string in the buffer is null terminated, so be
sure there is room for the null ( i.e. the actual size of
the buffer must be at least 1 byte larger than the size
passed to the routine).  If the string read completes because
the buffer filled (as far as the QIO was concerned), there
MUST be room for the terminating null!
.s
ttice() reads one character echoing it on on the terminal.
The actual character read is returned with the hi byte
cleared, or a 0 is returned if there was an error.
.s
tticne() reads one character without echoing it on the terminal.
The actual character read is returned with the hi byte
cleared, or a 0 is returned if there was an error.
.s
The ttice() and tticne() routines do NOT clear bit 7 (usually
the parity bit).
.s
The single character input routines ttice() and tticne()
use the IO.RAL function, so that any 8-bit character may be
read without trapping and interpretation by the terminal
driver.  The exceptions to this are _^S and _^Q, which
are trapped by the ttic(n)e() routine(s) and used to manually
synchronize the terminal driver.
.s
It is possible to get into a lockup or overrun condition with
terminals which automatically generate _^S and _^Q to
synchronize with the host computer when using the IO.RAL,
since both of these characters are passed to the program.
.s
For instance, suppose the program sends something which
causes the terminal to send back a _^S, and then posts an
IO.RAL.  If the _^S arrives before the RAL gets queued to
the terminal driver, the _^S will cause terminal output to
be suspended, since the terminal driver has no reason to
do anything special with the _^S.  Now the RAL arrives and
shortly thereafter the terminal sends a _^Q, expecting
terminal output to be resumed.  But instead, the terminal
driver faithfully passes the _^Q to the program. Meanwhile,
the terminal driver still has output suspended, and the
terminal is never going to generate another _^Q on it's own.
The terminal will appear to have mysteriously locked up.
.s
This and other scenarios make it wise to trap _^S and _^Q
within the CS library character input routines and issue
the appropriate SF.SMC function to suspend or resume terminal
output. The effect is to make the IO.RAL look like "read pass
almost all" (except for _^S and _^Q).  After processing the
_^S or _^Q, the routine issues another read without returning
to the caller.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########ttout###Terminal output
.pg
.hl 1 ^&Terminal output\&
.s 2
.c ;*********
.c ;* ttout *
.c ;*********
.s 2
.lm +8.nf
.i -8;Usage
.s
ttosf(buffer);
char *buffer;	/* Address of string */
.s
ttosnf(buffer);
char *buffer;	/* Address of string */
.s
ttocf(ch);
char ch;	/* Character to buffer */
.s
ttocnf(ch);
char ch;	/* Character to buffer */
.s
.s
.i -8;Description
.s
.fill
ttosf() buffers the passed character string into an internal
user output buffer and flushes the full contents of the buffer
to the terminal.
.s
ttosnf() buffers the passed character string into an internal user
output buffer. The buffer is not flushed unless it fills up. In
which case, the buffer is flushed and any remaining characters
are buffered. The capacity of the internal output buffer is 132.
characters, unless the user makes a call to the outbuf() routine
and resets the output buffer size.
.s
ttocf() buffers the passed character into an internal user output
buffer and flushes the contents of the output buffer to the terminal.
.s
ttocnf() buffers the passed character into an internal user output
buffer. The buffer is not flushed unless it fills up.
.s
All of the terminal output routines return a 1 if successful and
a 0 if an error occurred.
.s
.lm -8.fill
.lm 8.rm 72.nhy

.st ########wrapup##Dummy routine called on exit
.pg
.hl 1 ^&Dummy routine called on exit\&
.s 2
.c ;**********
.c ;* wrapup *
.c ;**********
.s 2
.lm +8.nf
.i -8;Usage
.s
wrapup()
.s
.i -8;Description
.s
.fill
This routine (which does nothing) is called on exit if the user's
program does not provide a wrapup() routine.
.s
Wrapup will be called only once.
.lm -8.fill
.lm 8.rm 72.nhy

